@@ -4,8 +4,8 @@ import urllib.parse
 from datetime import datetime
 from timeit import default_timer as timer
 
-from sqlalchemy import Column, Table, MetaData, create_engine, DateTime
-
+import sqlalchemy
+from sqlalchemy import Column, Table, MetaData, create_engine, DateTime, func, literal_column
 from bakplane.bakplane_pb2 import (
     ResolveResourcePathResponse,
     ResolveResourceIntentResponse,
@@ -323,17 +323,25 @@ def __build_read_query(
     else:
         blocks.columns = [a.c.get(x) for x in r.columns if x in a.c]
 
+        if r.partition_num is not None:
+            if r.partition_column_name not in a.c:
+                raise RuntimeError('Could not identify partition column.')
+
+            index_expression = a.c.knowledge_hash % r.partition_num
+            blocks.columns.append(index_expression.label('partition'))
+
         if r.additional_constraints is not None:
             blocks.constraints.append(r.additional_constraints(blocks))
 
         if r.effective_dating is not None:
-            blocks.constraints.append(
-                a.c.effective_start_dt <= blocks.mapping_table.c.effective_end_dt
-            )
+            if r.effective_dating_hint is None:
+                blocks.constraints.append(
+                    a.c.effective_start_dt <= blocks.mapping_table.c.effective_end_dt
+                )
 
-            blocks.constraints.append(
-                a.c.effective_end_dt >= blocks.mapping_table.c.effective_start_dt
-            )
+                blocks.constraints.append(
+                    a.c.effective_end_dt >= blocks.mapping_table.c.effective_start_dt
+                )
         else:
             blocks.constraints.append(
                 a.c.valid_start_dt <= blocks.mapping_table.c.effective_end_dt
@@ -384,7 +392,7 @@ def __build_read_query(
         res.compile(dialect=dialect, compile_kwargs={"literal_binds": True}, )
     )
 
-    return query
+    return query.replace('%%', '%')
 
 
 def write_fn(
@@ -410,19 +418,34 @@ def write_fn(
 
 
 def __read_query(
-        spark, url: str, driver: str, query: str, user: str, password: str
+        spark, url: str, driver: str, query: str, user: str, password: str, partition_num: int
 ):
     start = timer()
 
-    df = (
-        spark.read.format("jdbc")
-            .option("url", url)
-            .option("query", query)
-            .option("user", user)
-            .option("password", urllib.parse.unquote(password))
-            .option("driver", driver)
-            .load()
-    )
+    if partition_num is None:
+        df = (
+            spark.read.format("jdbc")
+                .option("url", url)
+                .option("query", query)
+                .option("user", user)
+                .option("password", urllib.parse.unquote(password))
+                .option("driver", driver)
+                .load()
+        )
+    else:
+        df = (
+            spark.read.format("jdbc")
+                .option("url", url)
+                .option("dbtable", f'({query}) s')
+                .option("user", user)
+                .option("password", urllib.parse.unquote(password))
+                .option("driver", driver)
+                .option("partitionColumn", "partition")
+                .option("lowerBound", 0)
+                .option("upperBound", partition_num - 1)
+                .option("numPartitions", partition_num)
+                .load()
+        )
 
     end = timer()
 
@@ -447,6 +470,7 @@ def read_all_fn(
         query=__build_read_all_query(r, res),
         user=res.context["username"],
         password=res.context["password"],
+        partition_num=r.partition_num
     )
 
 
@@ -466,4 +490,5 @@ def read_fn(
         query=__build_read_query(r, res, dialect, store_helper),
         user=res.context["username"],
         password=res.context["password"],
+        partition_num=r.partition_num,
     )
