@@ -12,6 +12,7 @@ from bakplane.extensions.base import (
     ReadAllRequest,
     ReadRequest,
 )
+
 from bakplane.extensions.spark import PluginEntry, WriteResponse
 from bakplane.extensions.spark.stores.jdbc import JDBCStoreHelper
 
@@ -49,7 +50,7 @@ def get_helper(
         database=database,
         schema_name=schema_name,
         driver="postgresql+psycopg2",
-        table_prefixes=['TEMPORARY'],
+        series_schema='series'
     )
 
 
